@@ -175,7 +175,7 @@ def convert_image(infile, outfile=None, artifacts=[], compressor=None, chunks=(-
         rc = csys.done()
         rc = IA.close()
 
-    print('incompatible components: ', diftypes)
+    if len(diftypes) > 0: print('incompatible components: ', diftypes)
 
     # if taylor terms are present, the chan axis must be expanded to the length of the terms
     if ttcount > len(mxds.chan): mxds = mxds.pad({'chan': (0, ttcount-len(mxds.chan))}, mode='edge')
@@ -200,7 +200,7 @@ def convert_image(infile, outfile=None, artifacts=[], compressor=None, chunks=(-
             # if the image set has taylor terms, loop through any for this artifact and concat together
             # pad the chan dim as necessary to fill remaining elements if not enough taylor terms in this artifact
             for ii in range(1, ttcount):
-                if ii < len(imagelist):
+                if (ii < len(imagelist)) and os.path.exists(imagelist[ii] + ext):
                     txds = convert_simple_table(imagelist[ii]+ext, outfile + '.temp', dimnames=dimorder, chunks=chunkorder, nofile=True)
                     ixds = xarray.concat([ixds, txds], dim='chan')
                 else:
